#
# Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors
#
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Integration test fixtures"""

import os
from .fixtures import Device, FileFactory
from c8ylp.env import loadenv

import pytest


@pytest.fixture
def c8ydevice():
    """Cumulocity integration test device fixture"""

    test_env = os.getenv("TEST_ENV_FILE", ".env")
    os.environ.setdefault("C8YLP_ENV_FILE", test_env)

    if os.path.exists(test_env):
        loadenv(test_env)

    yield Device(
        device=os.getenv("TEST_DEVICE", ""), ssh_user=os.getenv("TEST_SSH_USER", "")
    )


@pytest.fixture
def file_factory(tmpdir):
    """File factory fixture"""
    yield FileFactory(dir=tmpdir)
