#
# Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors
#
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Test plugin shell handling"""

import pytest

from c8ylp.cli.plugin import format_wsl_path


@pytest.mark.parametrize(
    "path,expected",
    [
        (
            "c:\\myscript\\ to my\\plugin.sh",
            "/mnt/c/myscript/ to my/plugin.sh",
        ),
        (
            "D:\\launcher.sh",
            "/mnt/d/launcher.sh",
        ),
    ],
)
def test_format_wsl_path(path, expected):
    """Test formating of a windows path to the WSL equivalent"""
    assert format_wsl_path(path) == expected
